/**
 * Copyright  2001 Columbia University in the City of New York.
 * All rights reserved.
 *
 * Columbia University will freely share software (understood to include
 * both source code and binaries) with educational institutions according
 * to the following terms:
 *
 * 1. The borrower agrees to indemnify and hold Columbia University
 * harmless against liability, damage, loss, cost, or expense arising
 * out of the use of the software.
 *
 * 2. Redistribution of the software must retain the above copyright
 * notice, disclaimer, and this list of conditions.
 *
 * 3. Any software distributed under the java class hierarchy of
 * "edu.columbia.*" may not be shared, redistributed or used in a
 * for-profit application without the  written consent of a representative
 * of Columbia University. (see contact information in the JA-SIG
 * Clearinghouse).
 *
 * 4. Columbia University does not warrant the software for suitability
 * of any purpose or reliability in any environment, and will provide no
 * service, support or maintenance of the software.
 *
 */

package  edu.columbia.ais.portal.channels.announcements;

import  edu.columbia.ais.portal.channels.announcements.*;
import  edu.columbia.ais.portal.utils.*;
import  org.jasig.portal.services.*;
import  org.jasig.portal.groups.*;
import  java.util.Hashtable;
import  java.util.Collection;
import  java.util.HashMap;
import  java.util.Enumeration;
import  java.util.Properties;
import  java.util.Vector;
import  java.util.Iterator;
import  java.util.ArrayList;
import  java.util.Set;
import  java.io.File;
import  java.io.FileInputStream;
import  java.util.Map.Entry;
import  java.util.Calendar;
import  java.sql.Timestamp;


/**
 * A class providing a variety of management tools for the announcements
 * system, including topic retrieval, topic store retrieval, and subscription
 * group retrieval
 *
 * @author Alex Vigdor
 * @version $Revision: 1.1 $
 */
public class TopicManager
        implements Constants {
    protected static TopicManager _instance = null;
    protected static HashMap topics = new HashMap();
    protected static HashMap stores = new HashMap();
    protected static ITopicStoreReadOnly[] storesArray;
    protected static ITopicStoreAuthority authority = null;
    protected static long refreshMillis;

    /** Creates new TopicManager */
    protected TopicManager () {
        try {
            Properties props = PropertiesLoader.getProperties(OWNER, PROPERTIES);
            String refreshPeriod = props.getProperty("Refresh.Period");
            int refreshFrequency = 12;          //default to 12 times an hour = 5 minutes
            if (props.getProperty("Refresh.Frequency").trim().length() > 0) {
                refreshFrequency = Integer.parseInt(props.getProperty("Refresh.Frequency"));
            }
            refreshMillis = calculateMillis(refreshPeriod, refreshFrequency);
            Enumeration s = props.propertyNames();
            ArrayList makeStores = new ArrayList();
            while (s.hasMoreElements()) {
                String propName = (String)s.nextElement();
                if (propName.startsWith("TopicStore.")) {
                    ITopicStoreReadOnly it = (ITopicStoreReadOnly)Class.forName(props.getProperty(propName)).newInstance();
                    it.setKey(propName);
                    stores.put(propName, it);
                    makeStores.add(it);
                }
            }
            storesArray = (ITopicStoreReadOnly[])makeStores.toArray(new ITopicStoreReadOnly[0]);
            String authKey = props.getProperty("TopicStoreAuthority");
            if (authKey != null && stores.get(authKey) != null) {
                this.authority = (ITopicStoreAuthority)stores.get(authKey);
            }
        } catch (Exception e) {
            LogService.log(LogService.ERROR, e);
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public long getRefreshMillis () {
        return  this.refreshMillis;
    }

    /**
     * put your documentation comment here
     * @param t
     * @return
     */
    public long getRefreshMillis (Topic t) {
        long rMillis = getRefreshMillis();
        if ((t.getRefreshFrequency() > 0) && (t.getRefreshPeriod().trim().length() > 0)) {
            rMillis = calculateMillis(t.getRefreshPeriod(), t.getRefreshFrequency());
        }
        return  rMillis;
    }

    /**
     * put your documentation comment here
     * @param period
     * @param frequency
     * @return
     */
    protected long calculateMillis (String period, int frequency) {
        long periodMillis = 3600000;            // default to hourly
        if (period.equals("daily")) {
            periodMillis = periodMillis*24;
        }
        if (period.equals("weekly")) {
            periodMillis = periodMillis*168;
        }
        if (period.equals("monthly")) {
            periodMillis = periodMillis*720;
        }
        if (period.equals("yearly")) {
            periodMillis = periodMillis*8760;
        }
        if (frequency > 0) {
            periodMillis = periodMillis/frequency;
        }
        return  periodMillis;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public synchronized static TopicManager instance () {
        if (_instance == null) {
            _instance = new TopicManager();
        }
        return  _instance;
    }

    /**
     * put your documentation comment here
     * @param ID
     * @return
     */
    public Topic getTopic (String ID) {
        if (topics.get(ID) == null) {
            this.getAllTopics();
        }
        refreshTopic((Topic)topics.get(ID), false);
        return  (Topic)topics.get(ID);
    }

    /**
     * put your documentation comment here
     * @param t
     * @param force
     */
    public void refreshTopic (Topic t, boolean force) {
        try {
            if (t != null) {
                long currentMillis = (Calendar.getInstance()).getTime().getTime();
                long rMillis = getRefreshMillis(t);
                Timestamp testTime = new Timestamp(currentMillis - rMillis);
                if (force || (t.getRetrievedAnnouncements() == null) || (t.getRetrievedAnnouncements().before(testTime))) {
                    //LogService.instance().log("Announcements::TopicManager::refreshing Topic "+t.getRetrievedAnnouncements().toString()+" vs "+testTime);
                    t = this.authority.refreshTopic(t);
                    ITopicStoreReadOnly tsro = (ITopicStoreReadOnly)this.stores.get(t.getTopicStore());
                    tsro.retrieveAllAnnouncements(t);
                    this.topics.put(t.getID(), t);
                    //LogService.instance().log(LogService.DEBUG,"Refreshed topic #"+(t.getID()).toString());
                }
                else {
                //LogService.instance().log("Announcements::TopicManager:: not refreshing topic "+t.getID());
                }
                if (t.getID() != null) {
                    t = (Topic)this.topics.get(t.getID());
                }
            }
        } catch (Exception e) {
            LogService.instance().log(LogService.ERROR, e);
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public ITopicStoreReadOnly[] getAllTopicStores () {
        return  this.storesArray;
    }

    /**
     * put your documentation comment here
     * @param key
     * @return
     */
    public ITopicStoreReadOnly getTopicStore (String key) {
        return  (ITopicStoreReadOnly)this.stores.get(key);
    }

    /**
     * put your documentation comment here
     * @return
     */
    public Topic[] getAllTopics () {
    
        // Delete from topics object topics that were deleted from database
                   // by another server.
                   Set dbTopicKeys = authority.getTopicIDs();
                   HashMap tmpMap = new HashMap();
                   tmpMap.putAll(topics);
                   Set topicKeys = tmpMap.keySet();
                   topicKeys.removeAll(dbTopicKeys);
                   if (!topicKeys.isEmpty()) {
                     Iterator i = topicKeys.iterator();
                     while (i.hasNext()) {
                       String topicKey = (String)i.next();
                       flushTopic((Topic)topics.get(topicKey));
                     }
                   }

        Topic[] rta = new Topic[0];
        try {
            Topic[] tsav = authority.retrieveNewTopicDefs();
            for (int i = 0; i < tsav.length; i++) {
                Topic t = tsav[i];
                //refreshTopic(t);
                LogService.instance().log(LogService.DEBUG, "Announcements::TopicManager::getAllTopics:: Filling slot #"
                        + t.getID().toString() + " with topic " + t.getTitle());
                if (t != null && t.getID() != null) {
                    ITopicStoreReadOnly st = getTopicStore(t.getTopicStore());
                    if (st != null) {
                        //st.retrieveAllAnnouncements(t);
                        topics.put(t.getID(), t);
                    }
                    else {
                        LogService.instance().log(LogService.DEBUG, "Announcements::TopicManager::getAllTopics:: topic "
                                + t.getID() + " has no store");
                    }
                }
            }
        } catch (Exception e) {
            LogService.instance().log(LogService.ERROR, e);
        }
        Collection tset = topics.values();
        Iterator it = tset.iterator();
        rta = (Topic[])topics.values().toArray(rta);
        //LogService.instance().log(LogService.DEBUG,"Announcements::TopicManager::getAllTopics::refreshing all topics");
        for (int j = 0; j < rta.length; j++) {
            refreshTopic(rta[j], false);
        }
        //LogService.instance().log(LogService.DEBUG,"Announcements::TopicManager::getAllTopics::finished refreshing all topics");
        return  rta;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public IEntityGroup[] getAllSubscriberGroups () {
        //LogService.instance().log(LogService.DEBUG,"Announcements::TopicManager::getAllSubscriberGroups - starting");
        IEntityGroup[] rga = new IEntityGroup[0];
        ArrayList sg = new ArrayList();
        Topic[] ta = (Topic[])topics.values().toArray(new Topic[0]);
        //LogService.instance().log(LogService.DEBUG,"Announcements::TopicManager::getAllSubscriberGroups - looping");
        for (int i = 0; i < ta.length; i++) {
            sg.add(ta[i].getSubscriberGroup());
        }
        //LogService.instance().log(LogService.DEBUG,"Announcements::TopicManager::getAllSubscriberGroups - done looping");
        rga = (IEntityGroup[])sg.toArray(rga);
        //LogService.instance().log(LogService.DEBUG,"Announcements::TopicManager::getAllSubscriberGroups - done");
        return  rga;
    }

    /**
     * put your documentation comment here
     * @param t
     */
    public void flushTopic (Topic t) {
        if (t != null) {
            topics.remove(t.getID());
            try {
                ViewXML.flushTopicByID(t.getID());
            } catch (Exception e) {
                LogService.instance().log(LogService.ERROR, "TopicManager.flushTopic() : could not flush viewXML cache for "
                        + String.valueOf(t.getID()) + " \n" + e);
            }
        }
        else {
            LogService.instance().log(LogService.ERROR, "TopicManager.flushTopic() : could not flush topic ID"
                    + String.valueOf(t.getID()));
        }
    }

    /**
     * put your documentation comment here
     * @return
     */
    public ITopicStoreAuthority getTopicStoreAuthority () {
        return  this.authority;
    }

    /**
     * put your documentation comment here
     * @param key
     * @param behavior
     * @return
     * @exception ClassNotFoundException
     */
    public boolean storeImplements (String key, String behavior) throws ClassNotFoundException {
        // get interface def for behavior
        boolean rval = false;
        Class bClass = Class.forName(packageBase + behavior);
        Class c = this.stores.get(key).getClass();
        // walk back up hierarchy to check interfaces
        while (c != null && rval == false) {
            // get interfaces for this class
            Class[] ia = c.getInterfaces();
            // is behavior among them?
            for (int i = 0; i < ia.length; i++) {
                if (ia[i] == bClass) {
                    rval = true;
                    break;
                }
                else {
                    Class[] ib = ia[i].getInterfaces();
                    for (int j = 0; j < ib.length; j++) {
                        if (ib[j] == bClass) {
                            rval = true;
                            break;
                        }
                    }
                }
            }
            c = c.getSuperclass();
        }
        //LogService.instance().log(LogService.DEBUG,"Checking if "+key+" implements "+behavior+" :"+rval);
        return  rval;
    }

    /**
     * put your documentation comment here
     * @return
     */
    public String[] getAllTopicKeys () {
        getAllTopics();
        return  (String[])topics.keySet().toArray(new String[0]);
    }

    /**
     * put your documentation comment here
     * @param key
     * @return
     */
    public String getTopicName (String key) {
        String name = "";
        Topic t = getTopic(key);
        if (t != null) {
            name = t.getTitle();
        }
        return  name;
    }
}

